#!/usr/bin/python

#  dir2ogg converts mp3, m4a, and wav files to the free open source OGG format. Oggs are
#  about 20-25% smaller than mp3s with the same relative sound quality. Your mileage may vary.

#  Keep in mind that converting from mp3 or m4a to ogg is a conversion between two lossy formats. 
#  This is fine if you just want to free up some disk space, but if you're a hard-core audiophile 
#  you may be dissapointed. I really can't notice a difference in quality with 'naked' ears myself.

#  This script converts mp3s to wavs using mpg123 then converts the wavs to oggs using oggenc. 
#  m4a conversions require faad. Id3 tag support requires pyid3lib for mp3s.
#  Scratch tags using the filename will be written for wav files (and mp3s with no tags!)

#  dir2ogg is Copyright(c) Darren Kirby 2003 - 2005. Released under the Artistic Licence:
#  http://www.opensource.org/licenses/artistic-license.php

#  Have fun kids!

import sys
import os, os.path
import string
import re
from fnmatch import filter
from getopt import gnu_getopt, GetoptError
sys.path.append('../lib')
sys.path.append('..')

import  LoadConfig
config = LoadConfig.LoadConfig()

#from MiscTools import *

def QuoteForPOSIX(string):
    return "\\'".join("'" + p + "'" for p in string.split("'"))

sys.stderr = sys.__stderr__
sys.stdout = sys.__stdout__

class NoHeaderError(Exception):
    pass

def info(msg=''):
    print "%s" % str(msg)
    #logger.debug( "%s" % msg)

def warn(msg=''):
    print "%s" % str(msg)
    #logger.debug( "%s" % msg)

def error(msg=''):
    print "*** %s ***" % str(msg)
    #logger.debug( "*** %s ***" % msg)

def debug(msg=''):
    print "*** %s ***" % str(msg)
    #logger.debug( "*** %s ***" % msg)

def returnDirs(root):
    mydirs = []
    for pdir, dirs, files in os.walk(root):
        if not pdir in mydirs:
            mydirs.append(pdir)
    return mydirs

class CleanUp:
    ''' 
    CleanUp: helper methods which tidy up a bit :)

    Note: wavs are deleted by default. use '-p' to save them.
    If you find more characters that break the script please
    send a bug report to dir2ogg@badcomputer.org 
    '''

    def filterShellKillers(self, string):
        '''we get it already escpaed when this script is called'''
        return string
        #filtered = "\\'".join("'" + p + "'" for p in string.split("'"))
	#debug( "ORIGINAL STRING: " + string)
	#debug( "FILTERED STRING: " + filtered)
        #return filtered

    def filterShellKillersOLD(self, ds):
        ''' Filter characters that break the script when fed to bash'''
        if re.search('\!',ds) != "None": cs = re.sub('\!', '', ds)
        if re.search(';', cs) != "None": cs = re.sub(';', '', cs)
        if re.search('\*', cs) != "None": cs = re.sub('\*', '', cs)
        if re.search('"', cs) != "None": cs = re.sub('"', '', cs)
        if re.search('&', cs) != "None": cs = re.sub('&', 'and', cs)
        if cs != ds:
            os.rename(ds, cs)
            if self.vf: warn('"%s" renamed "%s"' % (ds,cs))
        return cs

    def escapeShell(self, songSpaces):
        ''' Convert spaces to underscores.'''
        song = string.replace(songSpaces, ' ', '_')
        os.rename(songSpaces, song)
        return song

    def removeSong(self, song):
        os.remove(song)


class Id3TagHandler:
    ''' 
    Class for handling ID3 tags.

    If there are no tags, defaults will
    be created using the filename as basis.
    '''

    def grabMP3Tags(self):
        ''' Grab tags from mp3.'''
        try: import ID3
        except ImportError:
            warn('You dont have pyid3lib installed...')
            warn('Scratch tags will be created using filenames.')
        try: 
            x = ID3.ID3(self.fileIn)

            for key, val in x.items():
		x.key = val

        except Exception, e:
            debug( "in grabMP3Tags() exception: %s" % str(e))

        try: self.artist = x.artist
        except AttributeError: self.artist = string.strip(self.fileIn[:-4]) 
        try: self.title = x.title
        except AttributeError: self.title = string.strip(self.fileIn[:-4])
        try: self.album = x.album
        except AttributeError: self.album = "n/a"
        try: self.year = str(x.year)
        except AttributeError: self.year = "n/a"
        try:
            c = x.comment
            self.comment = "Comment=" + str(c)
        except ValueError: self.comment = "Comment=molested by dir2ogg"
        try:
            g = x.genre
            self.genre = g
        except ValueError: self.genre = "255" # 255 = 'unknown'
        try:
            self.track = x.track
            if len(self.track) == 2: self.track = str(self.track[0]) + "," + str(self.track[1])
            else: self.track = str(self.track[0])
        except: self.track = ""
        return self.artist, self.title, self.album, self.year, self.comment, self.genre, self.track

    def listIfVerbose(self):
        info('Id3 tags I will write:')
        info('Artist: ' + self.artist)  
        info('Title: ' + self.title) 
        info('Album: ' + self.album)
        info('Year: ' + self.year)
        info('Comment: ' + self.comment[8:])
        info('Genre: ' + self.genre)
        info('Track Num: ' + str(self.track))

#class Convert(Id3TagHandler, CleanUp):
class Convert(Id3TagHandler):
    ''' 
    Base conversion Class.

    __init__ creates some useful attributes,
    grabs the id3 tags, and sets a flag to remove files.
    Methods are the conversions we can do
    '''

    def __init__(self, fileIn, fileOgg=None):
        self.fileIn       = fileIn
        self.fileBasePath = os.path.splitext(fileIn)[0]
        self.fileBase     = os.path.basename(self.fileBasePath)
        self.fileWav      = config.spoolPath + str(self.fileBase) + '.wav'
        if fileOgg is None:
            self.fileOgg  = self.fileBasePath + '.ogg'
        else:
            self.fileOgg  = fileOgg
        self.tags         = self.grabMP3Tags()
        self.quality      = 10

	debug( self.tags)
        
    def getBitrate(self):
        from mutagen.mp3 import MP3
        audio = MP3(self.fileIn)
        try: bitrate = audio.info.bitrate / 1000
        except: bitrate = 192
	debug( "GOT BITRATE: " + str(bitrate))

	return int(bitrate)

    def mp3ToWav(self):
        ''' Convert mp3 -> wav.'''

        info(''' Converting from mp3 to wav.  Output from mpg123 @ bitrate: ''')
        cmd = 'mpg123 -w %s %s' % (QuoteForPOSIX(self.fileWav),QuoteForPOSIX(self.fileIn))
	debug( "----------------------------------------------")
	debug( "DECODING MP3 to WAV, USING COMMAND: " + cmd)
        es = os.system(cmd)
	debug( "----------------------------------------------")

        if es != 0:
            error('mpg123 error!')
            error('Decoding of "%s" failed. Corrupt mp3?' % (self.fileIn))
            self.r3 = 0 #  don't remove the mp3

    def wavToOgg(self):
        ''' Convert wav -> ogg.'''

        #if self.vf: self.listIfVerbose()
        info(''' Converting from wav to ogg.  Output from oggenc: ''')
        song_bitrate = self.getBitrate()
        
        es = 'oggenc'
    	
        if(len(str(self.tags[0])) > 0): es += ' -a %r' % self.tags[0]
        if(len(str(self.tags[1])) > 0): es += ' -t %r' % self.tags[1]
        if(len(str(self.tags[2])) > 0): es += ' -l %r' % self.tags[2]
        if(len(str(self.tags[3])) > 0): es += ' -d %r' % self.tags[3]
        if(len(str(self.tags[4])) > 0): es += ' -c %r' % self.tags[4]
        if(len(str(self.tags[5])) > 0): es += ' -G %r' % self.tags[5]
        if(len(str(self.tags[6])) > 0): es += ' -N %r' % self.tags[6]
        if(len(str(self.fileWav)) > 0): es += ' %r'    % self.fileWav

        debug( "[*] Tags will be: " + str(self.tags))

        if(1): es+= " -o %s" % QuoteForPOSIX(self.fileOgg)
                
    	#es = 'oggenc -b%i -a "%s" -t "%s" -l "%s" -d "%s" -c "%s" -G "%s" -N "%s" "%s"' % \
                #     (int(self.bitrate()), self.tags[0], self.tags[1], self.tags[2], self.tags[3], self.tags[4], self.tags[5], self.tags[6], self.songwav)

        if (song_bitrate > 0): esB = es + ' -b%i' % int(song_bitrate)
        else: esB = es + ' -q%i' % int(self.quality)

        import re
        esB = re.sub("\\\\x00", "", esB)
        debug( "[*] executing: %s " % esB)
        #try: enc = os.system(es)
	esB += " 2>&1"

	sys.stderr = sys.stdout
        enc = os.popen(esB)
        try: ret = str(enc.readlines()[4]).strip('\n')
        except: ret = "No error from mpeg decoder it seems"
        enc.close()

        if ret == "Mode initialisation failed: invalid parameters for bitrate":
	    print "* Bitrate encoding failed.."
            esB = es + ' -q%i' % int(self.quality)
	    print "* Trying encoding using quality instead of bitrate with command: %s" % esB
            esB += " 2>&1"

            enc = os.popen(esB)
            print enc.readlines()
            try: ret = str(enc.readlines())
            except: ret = "No error from mpeg decoder it seems"
            print "DECODING RESULT: %s" % ret
            enc.close()

        #except TypeError, v:
	if (0):
            warn('There seems to be something wrong with the tags, trying a modest subset... %s' % v)
            try:
                if (song_bitrate > 0):
                    es = os.system('oggenc -b%i -a "%s" -t "%s" -l "%s" "%s"' % \
                        (int(self.bitrate), self.tags[0], self.tags[1], self.tags[2],self.fileWav))
                else:
                    es = 'oggenc -q%i -a "%s" -t "%s" -l "%s" "%s"' % \
                         (int(self.quality), self.tags[0], self.tags[1], self.tags[2],self.fileWav)
                enc = os.system(es)
            except:
                warn('Still borked!!! Trying a minimal subset...')
                try:
                    es = 'oggenc -q%i -a "%s" -t "%s" "%s"' % (int(self.quality), self.tags[0], self.tags[1], self.fileWav)
                    enc = os.system(es)
                except:
                    warn('Sorry. No tags for you')
                    es = 'oggenc -q4 "%s"' % self.fileWav
                    enc = os.system(es)

        if enc != 0:
            error('oggenc error!')
            error('Encoding of "%s" failed.' % self.fileWav)

        info('Deleting temp file: %s' % self.fileWav)
        os.unlink(self.fileWav)

def showBanner():
    print '''dir2ogg Version 0.9.2 (proud to be released before Windows 'Vista'). Use at your own risk.)
Written by Darren Kirby :: d@badcomputer.org :: http://badcomputer.org/unix/dir2ogg/
Released under the Artistic License.
                                              * hacked by rama *
    '''

def getType(file):
    return os.path.splitext(file)[1]

def main(fileIn, fileOgg):
    showBanner()

    type = getType(fileIn)
    debug( "fileIn is: %s" % fileIn)
    debug( "type is: %s" % type)

    if type.lower() == '.mp3':
        debug( "converting mp3 to ogg")
        x = Convert(fileIn, fileOgg)
        x.mp3ToWav()
        x.wavToOgg()
    if type.lower() == '.m4a':
        debug( "converting m4a to ogg")
	sys.exit(0)
        x = Convert(fileIn)
        x.m4aToWav()
        x.wavToOgg()
    if type.lower() == '.wav':
        debug( "converting wav to ogg")
        x = Convert(fileIn, fileOgg)
        x.wavToOgg()

if __name__ == '__main__':
    if len(sys.argv) > 1: fileIn = sys.argv[1]
    else: debug( "* ERROR: you must give an input file")

    if len(sys.argv) > 2: fileOgg = sys.argv[2]
    else: fileOgg = None

    main(fileIn, fileOgg)

# dir2ogg version 0.9.2
