__doc__ = """Python MPEG Audio Information Module."""
__version__ = '0.3 20042001'  

#
#   Python MPEG Information Module
#
#   (c)Eduardo Roldan, Under the GPL License
#   e-mail: trazor@adinet.com.uy
#
#########################################################
# Adapted  by Marc Lindahl, still under GPL             #
# 4/20/01                                               #
# changed to operate on a string, instead of a file.    #
# use dictionaries for id3 params                       #
#                                                       #
#########################################################

#   Check the included READMEmp3info.txt for instructions (no many '#' comments here)

import string


_mpeg_strings = ['2.5', '', '2', '1']
_layers = ['', 'III', 'II', 'I']
_bitrates_mpeg1 = [ [],
                   [-1, 32, 40, 48, 56, 64, 80, 96, 112, 128, 160, 192, 224, 256, 320],
                   [-1, 32, 48, 56, 64, 80, 96, 112, 128, 160, 192, 224, 256, 320, 384],
                   [-1, 32, 64, 96, 128, 160, 192, 224, 256, 288, 320, 352, 384, 416, 448]
                 ]
                  # !! -1 = free bitrate
_bitrates_mpeg2 = [ [],
                   [-1, 8, 16, 24, 32, 40, 48, 56, 64, 80, 96, 112, 128, 144, 160],
                   [-1, 8, 16, 24, 32, 40, 48, 56, 64, 80, 96, 112, 128, 144, 160],
                   [-1, 32, 48, 56, 64, 80, 96, 112, 128, 144, 160, 176, 192, 224, 256]
                 ]
_frequency_strings = [ [11025, 12000, 8000],
                      [],
                      [22050, 24000, 16000,],
                      [44100, 48000, 32000,]
                    ]
_channel_mode_strings= ['Stereo', 'Joint Stereo', 'Dual Channel', 'Single Channel']

InvalidHeaderError = 'Can\'t read an invalid header'

class xxxNoHeaderError(Exception):
     def __init__(self, value):
         self.value = value
     def __str__(self):
         return repr(self.value)

class Error(Exception):
    """Base class for exceptions in this module."""
    pass

class NoHeaderError(Error):
    """
    Exception raised for errors in the input.
    """

    def __init__(self, message):
        self.message = message


NoTagError = 'No id3 tag present'

#Extracted from mpg123 0.59q
_genre_list = [
              "Blues", "Classic Rock", "Country", "Dance", "Disco", "Funk", "Grunge", "Hip-Hop",
              "Jazz", "Metal", "New Age", "Oldies", "Other", "Pop", "R&B", "Rap", "Reggae", "Rock",
              "Techno", "Industrial", "Alternative", "Ska", "Death Metal", "Pranks", "Soundtrack",
              "Euro-Techno", "Ambient", "Trip-Hop", "Vocal", "Jazz+Funk", "Fusion", "Trance",
              "Classical", "Instrumental", "Acid", "House", "Game", "Sound Clip", "Gospel", "Noise",
              "Alt", "Bass", "Soul", "Punk", "Space", "Meditative", "Instrumental Pop",
              "Instrumental Rock", "Ethnic", "Gothic", "Darkwave", "Techno-Industrial",
              "Electronic", "Pop-Folk", "Eurodance", "Dream", "Southern Rock", "Comedy", "Cult",
              "Gangsta Rap", "Top 40", "Christian Rap", "Pop/Funk", "Jungle", "Native American",
              "Cabaret", "New Wave", "Psychedelic", "Rave", "Showtunes", "Trailer", "Lo-Fi",
              "Tribal", "Acid Punk", "Acid Jazz", "Polka", "Retro", "Musical", "Rock & Roll",
              "Hard Rock", "Folk", "Folk/Rock", "National Folk", "Swing", "Fast-Fusion", "Bebob",
              "Latin", "Revival", "Celtic", "Bluegrass", "Avantgarde", "Gothic Rock",
              "Progressive Rock", "Psychedelic Rock", "Symphonic Rock", "Slow Rock", "Big Band",
              "Chorus", "Easy Listening", "Acoustic", "Humour", "Speech", "Chanson", "Opera",
              "Chamber Music", "Sonata", "Symphony", "Booty Bass", "Primus", "Porn Groove",
              "Satire", "Slow Jam", "Club", "Tango", "Samba", "Folklore", "Ballad", "Power Ballad",
              "Rhythmic Soul", "Freestyle", "Duet", "Punk Rock", "Drum Solo", "A Cappella",
              "Euro-House", "Dance Hall", "Goa", "Drum & Bass", "Club/House", "Hardcore", "Terror",
              "Indie", "BritPop", "Negerpunk", "Polsk Punk", "Beat", "Christian Gangsta Rap",
              "Heavy Metal", "Black Metal", "Crossover", "Contemporary Christian",
              "Christian Rock", "Merengue", "Salsa", "Thrash Metal", "Anime", "JPop", "Synthpop",""
             ]


class mp3info:
       """  Extract and insert MP3 meta information """

       ### Id3 ###

       def __init__(self, file=''):
            #read-only info (because it's in EVERY frame of the file)
            self.header = {'mpeg':'', 'layer':'', 'protection':'',
                    'bitrate':0, 'frequency':0,
                    'channel_mode':'', 'copyright':'',
                    'original':'', 'length':[0, 0, 0],
                    'framelength':0}
            #changeable info (at end of the file)
            self.id3 = {'genre':'', 'genre_no':0, 'songname':'', 'artist':'',
                    'album':'', 'year':0, 'comment':'',
                    'filesize':len(file), 'genre_no':255}
            self.data = file
            try:
                self.read_tag()
            except NoTagError:
                self.id3 = {'genre':'', 'genre_no':0, 'songname':'', 'artist':'',
                    'album':'', 'year':0, 'comment':'',
                    'filesize':len(file)}
            if file != '':
                self._initheader()  #no empty error if just constructing before using               

                    
       def __call__(self, file=None):
            """ basically same as init, reads file and strips tags, etc., don't catch excepts call with no arg to re-read current file """
            if file != None:
                self.data = file
            self.read_tag()
            self._initheader()
 


       def mp3_file(self, file=None, dict=None):   #call w arg to update file (w/opt. dict), w/o to use most recent file
            """ output an mp3 file with updated id3 tags """
            if file != None:
                self.data = file
                self.read_tag()
                self._initheader()
            if dict != None:
                self.set_id3(dict)

            return  string.join([self.data[:-128], 'TAG',
                string.ljust(self.id3['songname'], 30)[:30],
                string.ljust(self.id3['artist'], 30)[:30],
                string.ljust(self.id3['album'], 30)[:30],
                string.rjust(self.id3['year'], 4)[-4:],
                string.ljust(self.id3['comment'], 30)[:30],
                chr(self.id3['genre_no'])], '')

       def mp3_tag(self, file=None, dict=None):   #call w arg to update file (w/opt. dict), w/o to use most recent file
            """ output just the last 128 bytes of an mp3 file (tag data) with updated id3 tags """
            if file != None:
                self.data = file
                self.read_tag()
                self._initheader()
            if dict != None:
                self.set_id3(dict)

            return  string.join(['TAG',
                string.ljust(self.id3['songname'], 30)[:30],
                string.ljust(self.id3['artist'], 30)[:30],
                string.ljust(self.id3['album'], 30)[:30],
                string.rjust(self.id3['year'], 4)[-4:],
                string.ljust(self.id3['comment'], 30)[:30],
                chr(self.id3['genre_no'])], '')


                
       def free_file(self):   #free data storage
            """ free data store (which could be large) """
            self.data = None

       def _initheader(self):
            self.sync_read_header(0, 1024)
            self.header['mpeg'] = self.get_mpeg_version()
            self.header['layer'] = self.get_layer()
            self.header['protection'] = self.get_protection()
            self.header['bitrate'] = `self.get_bitrate()`
            self.header['frequency'] = `self.get_frequency()`
            self.header['channel_mode'] = self.get_channel_mode()
            self.header['copyright'] = self.get_copyright()
            self.header['original'] = self.get_original()
            self.header['length'] = string.zfill(self.get_length()[1], 2) + ':' + string.zfill(self.get_length()[2], 2)
            self.header['framelength'] = `self.get_framelength()`
            self.id3.update(self.header)    #put readonly info into id3 dict


 
       def read_tag(self):
            """ update id3 info dictionary """
            self.id3['filesize'] = len(self.data)
            if self.id3['filesize'] <= 128:
                raise NoTagError
            if self.data[-128:-125] == 'TAG':
                self.id3['songname'] = self.data[-125:-95]
                self.id3['artist'] = self.data[-95:-65]
                self.id3['album'] = self.data[-65:-35]
                self.id3['year'] = self.data[-35:-31]
                self.id3['comment'] = self.data[-31:-1]
                self.id3['genre_no'] = ord(self.data[-1])
                self.id3['genre'] = self.get_genre()
                return 1
                
            else:
                raise NoTagError


       def get_genre(self, g = None):
            """ get genre """
            if g == None:
                g =  self.id3['genre_no']
            try:
                return _genre_list[g]
            except IndexError:
                return 'Unknown'

       def get_genre_list(self):        ##MSL added 4/20/01
            """ get list of genres """
            return _genre_list
            

       def get_id3(self):               ##MSL added 4/20/01
            """ get id3 info dictionary """
            return self.id3
            
            
       ### Set method ###               ##MSL added 4/20/01
             
       def set_id3(self, id3):
            """ set id3 info dictionary """
            self.id3.update(id3)
            self.id3['genre'] = self.get_genre()
            self.id3.update(self.header)    #put readonly info into id3 dict
            return self.id3
      
        
       ### Header ###

       def sync_read_header(self, offset = 0, depth = 0):

            if depth == 0 or depth > len(self.data):
                depth = len(self.data)

            _seek = offset       #MSL 04/21/01 hackery - since original code dealt with filesystem (open, seek, etc)

            while 1:
                    while _seek < depth:

                           if ord(self.data[_seek]) != 0xff:
                               _seek = _seek + 1
                               break

                           _seek = _seek + 1
                           _second_byte = ord(self.data[_seek])
                           _seek = _seek + 1
                           if _second_byte >> 5 != 0x07:
                               _seek = _seek - 1
                               break

                           self._sync_byte = 255
                           self._mpeg_version = (_second_byte & 0x18) >> 3  # B
                           self._layer = (_second_byte & 0x06) >> 1         # C
                           _third_byte = ord(self.data[_seek])
                           _seek = _seek + 1
                           self._bitrate = (_third_byte & 0xf0) >> 4        # E
                           self._frequency = (_third_byte & 0x0c) >> 2      # F
                           self._padding = (_third_byte & 0x02) >> 1        # G
                           _fourth_byte = ord(self.data[_seek])
                           _seek = _seek + 1
                           self._emphasis = (_fourth_byte & 0x03)           # M

                           try:
                               self.check_header()
                           except InvalidHeaderError:
                               _seek = _seek - 3
                               break
                           ctuple1 = self._mpeg_version, self._layer, self._frequency, self._emphasis
                           step_1_ = self.get_framelength()
                           _seek = _seek + step_1_ - 4


                           try:
                               self.read_header(_seek)
                           except InvalidHeaderError:
                                _seek = _seek - (step_1_ + 3)
                                break
                           ctuple2 = self._mpeg_version, self._layer, self._frequency, self._emphasis

                           if ctuple1 == ctuple2:
                                return _seek - (step_1_+4)
                           else:
                                _seek = _seek - (step_1_ + 3)
                                break
                    else:
                         raise NoHeaderError("No header present, or could not be read")

       def read_header(self, offset = 0):

            # AAAAAAAA AAABBCCD EEEEFFGH IIJJKLMM

            _seek = offset       #MSL 04/21/01 hackery - since original code dealt with filesystem (open, seek, etc)

            self._sync_byte = ord(self.data[_seek])
            _seek = _seek + 1
            
            _second_byte = ord(self.data[_seek])
            _seek = _seek + 1
            self._mpeg_version = (_second_byte & 0x18) >> 3        # B
            self._layer = (_second_byte & 0x06) >> 1               # C
            self.protection = (_second_byte & 0x01)               # D

            _third_byte = ord(self.data[_seek])
            _seek = _seek + 1
            self._bitrate = (_third_byte & 0xf0) >> 4           # E
            self._frequency = (_third_byte & 0x0c) >> 2         # F
            self._padding = (_third_byte & 0x02) >> 1           # G
            self.private = (_third_byte & 0x01)                # H

            _fourth_byte = ord(self.data[_seek])
            _seek = _seek + 1
            self.channel_mode = (_fourth_byte & 0xc0) >> 6     # I
            self.mode_ext = (_fourth_byte & 0x30) >> 4         # J
            self.copyright = (_fourth_byte & 0x08) >> 3        # K
            self.original = (_fourth_byte & 0x04) >> 2         # L
            self._emphasis = (_fourth_byte & 0x03)              # M

            self.check_header()
            return 1

       def check_header(self):
            if self._sync_byte != 255 or self._mpeg_version == 1 or self._layer == 0\
            or self._bitrate == 15 or self._frequency == 3 or self._emphasis == 2 :
               raise InvalidHeaderError

       def get_mpeg_version(self):
            return _mpeg_strings[self._mpeg_version]

       def get_layer(self):
            return _layers[self._layer]

       def get_protection(self):
            if self.protection:
                return 'No'
            else:
                return 'Yes'

       def get_bitrate(self):
            if self._mpeg_version == 3:
                  return _bitrates_mpeg1[self._layer][self._bitrate]
            elif self._mpeg_version == 2 or self._mpeg_version == 0:
                  return _bitrates_mpeg2[self._layer][self._bitrate]

       def get_frequency(self):
            return _frequency_strings[self._mpeg_version][self._frequency]

       def get_channel_mode(self):
            return _channel_mode_strings[self.channel_mode]

       def get_copyright(self):
            if self.copyright:
                return 'Yes'
            else:
                return 'No'

       def get_original(self):
            if self.original:
                return 'Yes'
            else:
                return 'No'

       def get_length(self):
            if self._bitrate == 0:
                return 0, 0, 0
            else:
                _const = self.get_bitrate() * 125
                _seconds = self.id3['filesize'] / _const
                _minutes = _seconds / 60
                _seconds_part = _seconds % 60
                return _seconds, _minutes, _seconds_part

       def get_framelength(self):
            if self._layer == 1  or self._layer == 2:
                return 144 * self.get_bitrate() * 1000 / self.get_frequency() + self._padding
            else:
                return 12 * self.get_bitrate() * 1000 / self.get_frequency() + self._padding
