#!/bin/env python
# -*- coding: utf-8 -*-

import MySQLdb
import  sys
sys.path.append('..')
import  LoadConfig

config = LoadConfig.LoadConfig()

from MiscTools import *

db = MySQLdb.connect(host=config.DBhost, user=config.DBuser, passwd=config.DBpass, db=config.DB)
db.set_character_set('utf8')

def GetLabels(searchString=''):
    sql  = "SELECT DISTINCT la.id, la.name, la.img_src, la.comments"
    sql += " FROM netjuke_labels la, netjuke_tracks tr"
    sql += " WHERE la.id=tr.la_id AND tr.report=0"
    sql += " AND la.exclude = 'f' AND tr.exclude = 'f'"
    sql += " AND la.name LIKE '%"
    sql += "%s" % searchString
    sql += "%'" + " ORDER BY la.name"

    cursor = db.cursor()
    cursor.execute(sql)
    result = cursor.fetchall()
    records =[] 
    for row in result:
        records.append({ 'id':row[0], 'name':uniconvert2(row[1]), 'img':row[2], 'info':row[3] })

    return records

def GetArtists(labelID):

    logger.debug( "* Getting ARTISTS for LABEL: %i" % labelID )

    sql = "/* Get ARTISTS for this LABEL */ \
            SELECT DISTINCT ar.id id, ar.name name, ar.img_src, ar.comments \
            FROM netjuke_artists ar, netjuke_tracks tr, netjuke_labels la, \
            netjuke_albums al \
            WHERE tr.la_id=la.id \
            AND ar.id=tr.ar_id \
            AND al.id=tr.al_id"
    sql += " AND la.id=%s" % labelID
    sql += " AND tr.report=0"
    sql += " AND ar.exclude = 'f' AND la.exclude = 'f' AND tr.exclude = 'f'"
    sql += " ORDER BY name"

    cursor = db.cursor()
    cursor.execute(sql)
    result = cursor.fetchall()
    records =[] 
    for row in result:
        records.append({ 'id':row[0], 'name':uniconvert2(row[1]), 'img':row[2], 'info':row[3] })

    return records

def GetAlbums(labelID, artistID):
    logger.debug( "* Getting ALBUMS for LABEL: %i and ARTIST: %i" % (labelID, artistID) )
    sql = "SELECT DISTINCT al.id id, al.name album, al.img_src, al.comments" \
        + " FROM netjuke_artists ar, netjuke_albums al, netjuke_tracks tr," \
        + " netjuke_labels la" \
        + " WHERE tr.ar_id=ar.id AND tr.al_id=al.id AND" \
        + " ar.id=" + str(artistID) \
        + " AND tr.la_id=la.id" \
        + " AND tr.la_id=" + str(labelID) \
        + " AND tr.report=0" \
        + " AND ar.exclude = 'f' AND al.exclude = 'f' AND la.exclude = 'f' AND tr.exclude = 'f'" \
        + " ORDER BY album" 
    
    #logger.debug( sql )

    cursor = db.cursor()
    cursor.execute(sql)
    result = cursor.fetchall()
    records =[] 
    for row in result:
        records.append({ 'id':row[0], 'name':uniconvert2(row[1]), 'img':row[2], 'info':row[3] })

    return records

def GetTracks(labelID, artistID, albumID):
    sql = "SELECT DISTINCT tr.id id, tr.name album, tr.location location, tr.img_src, tr.comments" \
        + " FROM netjuke_artists ar, netjuke_albums al, netjuke_tracks tr," \
        + " netjuke_labels la" \
        + " WHERE tr.ar_id=ar.id AND tr.al_id=al.id AND tr.la_id=la.id" \
        + " AND tr.la_id=" + str(labelID) \
        + " AND ar.id=" + str(artistID) \
        + " AND tr.al_id=" + str(albumID) \
        + " AND tr.report=0" \
        + " AND ar.exclude = 'f' AND al.exclude = 'f' AND la.exclude = 'f' AND tr.exclude = 'f'" \
        + " ORDER BY album" 
    
    #logger.debug( sql )

    cursor = db.cursor()
    cursor.execute(sql)
    result = cursor.fetchall()
    records =[] 
    for row in result:
        records.append({ 'id':row[0], 'name':uniconvert2(row[1]), 'location':'/usr/local/media/'+row[2], 'img':row[3], 'info':row[4] })

    return records

def AddToPlaylist(trackID, playlistID):
    sql = "INSERT IGNORE INTO netjuke_plists_tracks SET tr_id=%s, pl_id=%s" % (trackID, playlistID)
    
    logger.debug( sql )

    cursor = db.cursor()
    cursor.execute(sql)

def GetPlaylist(playlistID):
    sql = "SELECT pl.tr_id, tr.name, tr.location" \
          " FROM netjuke_plists_tracks pl, netjuke_tracks tr " \
          " WHERE pl.pl_id = %i AND pl.tr_id=tr.id" \
          " ORDER BY pl.id DESC" % playlistID
    
    logger.debug( sql )

    cursor = db.cursor()
    cursor.execute(sql)
    result = cursor.fetchall()
    records =[]
    if len(result) == 0:
        logger.info("No items in playlist")
        result = [ ['', 'Playlist is empty', '', ''] ]
    for row in result:
        records.append({ 'id':row[0], 'name':str(row[1]), 'location':'/usr/local/media/'+row[2], 'img':str(row[0]) })

    return records

def RemoveFromPlaylist(playlistID, trackID):
    sql = "DELETE FROM netjuke_plists_tracks WHERE tr_id = '%i' AND pl_id='%i'" % (playlistID, trackID)
    logger.debug( sql )
    cursor = db.cursor()
    cursor.execute(sql)


def CreatePlaylist():
    '''Create a new playlist. Returns the ID of the new playlist.'''

    timestamp = datetime.datetime.now().strftime("%Y%m%d%H%M%S")

    cursor = db.cursor()
    cursor.execute("INSERT INTO netjuke_plists SET title='New Playlist', us_email=%s, created=now()", (timestamp))
    playlistID = db.insert_id()

    return playlistID
