#!/bin/env python
# -*- coding: utf-8 -*-

from engine import Game, Scene
import pygame
from pygame.locals import *
import math
import time

import Database

import pygame
import pygame.joystick
import pygame.display

import os, os.path
from MiscTools import *
import urllib
import random

from Browser import *

pygame.display.init()
pygame.joystick.init() #initialize joystick module
pygame.joystick.get_init() #verify initialization (boolean)

FRAMERATE=30
WIDTH=800
HEIGHT=600
WIDTH=1024
HEIGHT=768
WHITE=(255,255,255)
BLACK=(0,0,0)
ORANGENESS=.7
BARLIGHT=(255,255*ORANGENESS,0)
BARDARK=(128,128*ORANGENESS,0)
BG_SELECTED=(10,10,10)
VISIBLES=15
FONTSIZE=60
ACELERACION=1.05
DELAY=.25
MAXSPEED=10

def load_image(name, colorkey=None):
    fullname = name
    try:
        image = pygame.image.load(fullname)
    except pygame.error, message:
        logger.debug( "Cannot load image:" + name )
        raise SystemExit, message
    image = image.convert()
    if colorkey is not None:
        if colorkey is -1:
            colorkey = image.get_at((0,0))
        image.set_colorkey(colorkey, RLEACCEL)
    return image, image.get_rect()

#--------------------------------------------------------------------
class FontCache:
    def __init__(self, face, size):
        self.face = face
        self.size = size
        self.cache = {}

        num_joysticks = pygame.joystick.get_count()
        if num_joysticks > 0:
            self.stick = pygame.joystick.Joystick(0)
            self.stick.init() # now we will receive events for the joystick

    #--------------------------------------------------------------------
    def get(self, scale):
        font = self.cache.get(scale)
        if font is None:
            font = self.cache[scale] = pygame.font.Font(self.face, int(round(self.size * scale)))
        return font

#--------------------------------------------------------------------
class Scroller(pygame.sprite.Sprite):
    def __init__(self, browser, surface, posX, bgcolor, fontsize, visibles):
        pygame.sprite.Sprite.__init__(self)

        self.items=[]

        # instantiate the browser object
        self.browser = browser
        # start browsing at the labels level
        self.list  = self.browser.getList(browser.itemID)
        self.image, self.rect = load_image('data/fist.bmp', -1)

        for record in self.list:
            self.items.append(record['name'])

        # select middle item to start
        #self.selected   = len(self.items)/2
        self.selected   = 0

        self.fontsize   = fontsize
        self.visibles   = visibles
        self.fontcache  = FontCache(None, self.fontsize)
        self.subiendo   = 0
        self.bajando    = 0
        self.velocidad  = 0
        self.mustRender = True
        self.imgpath    = ""
        self.info       = ""
        self.posX       = posX
        self.bgcolor    = bgcolor

        # do the initial render
        self.surface = surface
 #       self.render(self.surface)

    #--------------------------------------------------------------------
    def Refresh(self):
        self.list  = self.browser.getList()
        self.Redraw()
        print "Refresh called"

    #--------------------------------------------------------------------
    def Redraw(self, index=-1):
        self.items=[]
        for record in self.list:
            self.items.append(record['name'])
        self.getSelected(self.list, index)
        self.mustRender = True

    #--------------------------------------------------------------------
    def getSelected(self, list, index=-1):
        if index > -1: self.selected = index
        else:
            #if self.browser.level != 'tracks': self.selected = 0
            self.selected = 0

    #--------------------------------------------------------------------
    def goUp(self):
        self.keystart = time.time()
        self.selected -= 1
        self.velocidad = -1.0
        self.mustRender = True
        self.itemID = self.list[self.selected]['id']

    #--------------------------------------------------------------------
    def goDown(self):
        self.keystart = time.time()
        self.selected += 1
        self.velocidad = 1.0
        self.mustRender = True
        try: self.itemID = self.list[self.selected]['id']
        except: logger.warn("Invalid index")

    #--------------------------------------------------------------------
    def stopMoving(self):
        self.velocidad = 0.0
        self.selected = int(round(self.selected))

    #--------------------------------------------------------------------
    def BrowseForward(self):
        # identify which item was selected to be browse
        self.itemID = self.list[self.selected]['id']
        if self.browser.level == 'labels':
            #self.labelName = self.list[self.selected]['name']
            print self.list[self.selected]['name']
        if self.browser.level != 'tracks' and self.browser.level != 'playlist_tracks':
            self.list = self.browser.getList(self.itemID, self.selected)
            self.Redraw()
        else:
            self.Play()

    #--------------------------------------------------------------------
    def BrowseBack(self):
        if self.browser.level != 'labels' and self.browser.level != 'playlist_tracks':
            goBack = self.browser.Back()
            self.list = goBack['list']
            self.Redraw(goBack['index'])

    #--------------------------------------------------------------------
    def AddToRemoveFromPlaylist(self, playlistID):
        if self.browser.level == 'tracks':
            logger.info( "Adding track to playlist" )
            Database.AddToPlaylist(self.list[self.selected]['id'], playlistID)
        elif self.browser.level == 'playlist_tracks':
            logger.info("Deleting track from playlist")
            Database.RemoveFromPlaylist(self.list[self.selected]['id'], playlistID)
        else: logger.info( "Not a track.. not adding to playlist" )

    #--------------------------------------------------------------------
    def Play(self):
        '''Play the file'''
        filepath = self.list[self.selected]['location']
        file = urllib.url2pathname(filepath)
        logger.debug( "Play file : " + file )
        try:
            pygame.mixer.music.load(file)
            self.playtime = 0
            pygame.mixer.music.play()
        except Exception, e:
            logger.error( "File not found: " + str(e) )

    #--------------------------------------------------------------------
    def Seek(self, direction):
        curpos = pygame.mixer.music.get_pos()
        if direction == 'fwd':
            self.playtime += float(curpos) / 1000
            newpos = self.playtime * 2
        elif direction == 'rew':
            self.playtime -= float(curpos) / 1000
            newpos = self.playtime / 2
        logger.debug( "Current playback to position: %s" % str(curpos) )
        logger.debug( "Seeking playback to position: %s" % str(newpos) )
        pygame.mixer.music.play(0, newpos)

    #--------------------------------------------------------------------
    def update(self):
        if self.mustRender:
            self.render()
            self.mustRender = False

    #--------------------------------------------------------------------
    def getLabel(self, number):
        number = int(round(number))
        if number < 0 or number >= len(self.items):
            return
        return self.items[number]

    #--------------------------------------------------------------------
    def render(self):
        self.surface.fill(self.bgcolor)

        def unaLinea(n, seleccionado=0):
            a = self.getLabel(self.selected+n)
            if a is None:
                return

            o = n * (0.7/self.visibles)
            j = n * (1.0/self.visibles)
            v = 255*(1-math.sin((math.fabs(j))*math.pi/2))
            if not seleccionado:
                color = (v,v*ORANGENESS,0) 
            if seleccionado:
                color = WHITE
            scale = 1-math.fabs(o)
            font = self.fontcache.get(scale)
            t = font.render( a, 1, color)
            deltay = n*self.fontsize*(1-math.fabs(o)) * 0.6
            #self.posX = random.randint(30,40)
            self.surface.blit(t, t.get_rect(left=self.surface.get_rect().left+self.posX, centery=self.surface.get_rect().centery+ deltay ) )

        if self.selected < 0:
            self.selected = 0
        if self.selected >= len(self.items):
            self.selected = len(self.items)-1

        for n in range(self.visibles, 0, -1):
            unaLinea(-n)
            unaLinea(n)
        unaLinea(0,1)

        self.surface.fill(BARDARK, self.surface.get_rect(left=10, width=10))
        barheight = 10
        barrect = self.surface.get_rect(left=10, width=10, height=barheight)
        ratio = float(self.selected) / len(self.items)
        barrect.top = (self.surface.get_rect().height-barheight) * ratio
        self.surface.fill(BARLIGHT, barrect)

        self.showRelatedImage((60,100), self.surface)
        #self.showImage("play.png", (500,650), self.surface)
        ####self.showText("Burn Station 2.0-beta", 48, 200, 20, (200, 10, 10), self.surface)
        ####self.showText("Browsing: " + self.browser.level, 36, 130, 50, (200, 50, 10), self.surface)
        #pygame.image.save(self.surface, "/tmp/bs2.0.png")

    #--------------------------------------------------------------------
    def showText(self, string, size, x, y, color, surface):
        font = pygame.font.Font(None, size)
        text = font.render(string, 1, color)
        textpos = text.get_rect()
        textpos.centerx = x
        textpos.centery = y
        surface.blit(text, textpos)

    #--------------------------------------------------------------------
    def showImage(self, imgfile, pos, surface):
        #self.image, self.rect = load_image(imgfile, -1)
        #self.surface.blit(self.image, pos)
        img = pygame.image.load(imgfile)
        self.surface.blit(img, pos)


    #--------------------------------------------------------------------
    def showRelatedImage(self, pos, surface):
        # show related image
        itemImg = ''
        if type(self.selected) is int:
            itemImg = self.list[self.selected]['img']
            if itemImg != '':
                imgsrc = ValidateImage(itemImg, self.browser.level)
                if imgsrc:
                    logger.debug( "loading image: " + imgsrc )
                    self.showImage(imgsrc, pos, self.surface)
        else: return

        #if img: self.imgpath = "/usr/local/media/" + img
        # display associated image
        #defaultImage = "../img/burnlogo.png"

    #--------------------------------------------------------------------
    def SetSelected(self):
        self.bgcolor = BG_SELECTED
        self.mustRender = True

    #--------------------------------------------------------------------
    def SetUnSelected(self):
        self.bgcolor = BLACK
        self.mustRender = True

    #--------------------------------------------------------------------
    def loop(self):
        if self.velocidad and time.time() > self.keystart + DELAY:
            self.selected = self.selected + self.velocidad
            if( math.fabs(self.velocidad) < MAXSPEED) :
                self.velocidad *= ACELERACION
            self.mustRender = True

#--------------------------------------------------------------------
